import { defineNuxtModule } from '@nuxt/kit';
import * as lodash from 'lodash-es';

const exculdeDefaults = [
  "wrapperValue",
  "wrapperToIterator",
  "wrapperReverse",
  "wrapperPlant",
  "wrapperNext",
  "wrapperLodash",
  "wrapperCommit",
  "wrapperChain",
  "wrapperAt",
  "templateSettings",
  "toIterator",
  "VERSION",
  "lodash",
  "value",
  "valueOf",
  "toJSON",
  "thru",
  "plant",
  "next",
  "default",
  "commit"
];

const module = defineNuxtModule({
  meta: {
    name: "nuxt-lodash",
    configKey: "lodash",
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  defaults: {
    prefix: "use",
    prefixSkip: ["is"],
    exclude: [],
    alias: []
  },
  setup(options, nuxt) {
    const names = [];
    const prefix = options.prefix || "";
    const aliasMap = new Map(options.alias);
    const exludes = [...options.exclude, ...exculdeDefaults];
    for (const [name] of Object.entries(lodash)) {
      if (!exludes.includes(name)) {
        const alias = aliasMap.has(name) ? aliasMap.get(name) : name;
        const as = (() => {
          const isPrefix = !options.prefixSkip.some((key) => alias.startsWith(key)) && prefix;
          return isPrefix ? prefix + lodash.upperFirst(alias) : alias;
        })();
        names.push({ name, as });
      }
    }
    nuxt.hook("vite:extend", ({ config }) => {
      var _a;
      config.optimizeDeps || (config.optimizeDeps = {});
      (_a = config.optimizeDeps).exclude || (_a.exclude = []);
      config.optimizeDeps.exclude.push("lodash-es");
    });
    nuxt.hook("autoImports:sources", (sources) => {
      sources.push({ names, from: "lodash-es" });
    });
  }
});

export { module as default };
